@@ -32,16 +32,12 @@
 #define _WIN32_WINNT 0x0500
 #define WINVER _WIN32_WINNT
 
-#include "gdk.h" /* gdk_rectangle_intersect */
-#include "gdkevents.h"
-#include "gdkpixmap.h"
-#include "gdkwindow.h"
-#include "gdkdisplay.h"
+#include "gdk.h"
 #include "gdkprivate-win32.h"
 #include "gdkinput-win32.h"
 
 #if defined(_MSC_VER) && (WINVER < 0x0500)
-#define GetAncestor(hwnd,what) _gdk_win32_get_ancestor_parent(hwnd)
+#define GetAncestor(hwnd,what) _gdk_win32_get_ancestor_parent (hwnd)
 
 static HWND
 _gdk_win32_get_ancestor_parent (HWND hwnd)
@@ -208,14 +204,6 @@ _gdk_win32_adjust_client_rect (GdkWindow *window,
   API_CALL (AdjustWindowRectEx, (rect, style, FALSE, exstyle));
 }
 
-void
-_gdk_win32_get_adjusted_client_rect (GdkWindow *window,
-				     RECT      *rect)
-{
-  GetClientRect (GDK_WINDOW_HWND (window), rect);
-  _gdk_win32_adjust_client_rect (window, rect);
-}
-
 static GdkColormap*
 gdk_window_impl_win32_get_colormap (GdkDrawable *drawable)
 {
@@ -228,8 +216,7 @@ gdk_window_impl_win32_get_colormap (GdkDrawable *drawable)
   if (!((GdkWindowObject *) drawable_impl->wrapper)->input_only && 
       drawable_impl->colormap == NULL)
     {
-      drawable_impl->colormap = 
-        gdk_screen_get_system_colormap (gdk_drawable_get_screen (drawable));
+      drawable_impl->colormap = gdk_screen_get_system_colormap (_gdk_screen);
       g_object_ref (drawable_impl->colormap);
     }
   
@@ -254,7 +241,7 @@ gdk_window_impl_win32_set_colormap (GdkDrawable *drawable,
   if (cmap)
     {
       /* XXX */
-      g_print("gdk_window_impl_win32_set_colormap: XXX\n");
+      g_print ("gdk_window_impl_win32_set_colormap: XXX\n");
     }
 }
 
@@ -337,7 +324,7 @@ _gdk_windowing_window_init (void)
   
   draw_impl->handle = GetDesktopWindow ();
   draw_impl->wrapper = GDK_DRAWABLE (private);
-  draw_impl->colormap = gdk_colormap_get_system ();
+  draw_impl->colormap = gdk_screen_get_default_colormap (_gdk_screen);
   g_object_ref (draw_impl->colormap);
   
   private->window_type = GDK_WINDOW_ROOT;
@@ -430,7 +417,7 @@ RegisterGdkClass (GdkWindowType wtype)
 	{
 	  wcl.lpszClassName = "gdkWindowToplevel";
 	  
-	  ONCE_PER_CLASS();
+	  ONCE_PER_CLASS ();
 	  klassTOPLEVEL = RegisterClassEx (&wcl);
 	}
       klass = klassTOPLEVEL;
@@ -442,7 +429,7 @@ RegisterGdkClass (GdkWindowType wtype)
 	  wcl.lpszClassName = "gdkWindowChild";
 	  
 	  wcl.style |= CS_PARENTDC; /* MSDN: ... enhances system performance. */
-	  ONCE_PER_CLASS();
+	  ONCE_PER_CLASS ();
 	  klassCHILD = RegisterClassEx (&wcl);
 	}
       klass = klassCHILD;
@@ -453,7 +440,7 @@ RegisterGdkClass (GdkWindowType wtype)
 	{
 	  wcl.lpszClassName = "gdkWindowDialog";
 	  wcl.style |= CS_SAVEBITS;
-	  ONCE_PER_CLASS();
+	  ONCE_PER_CLASS ();
 	  klassDIALOG = RegisterClassEx (&wcl);
 	}
       klass = klassDIALOG;
@@ -464,7 +451,7 @@ RegisterGdkClass (GdkWindowType wtype)
 	{
 	  wcl.lpszClassName = "gdkWindowTemp";
 	  wcl.style |= CS_SAVEBITS;
-	  ONCE_PER_CLASS();
+	  ONCE_PER_CLASS ();
 	  klassTEMP = RegisterClassEx (&wcl);
 	}
       klass = klassTEMP;
@@ -499,7 +486,6 @@ gdk_window_new_internal (GdkWindow     *parent,
   GdkWindowObject *private;
   GdkWindowImplWin32 *impl;
   GdkDrawableImplWin32 *draw_impl;
-  GdkScreen *screen;
   GdkVisual *visual;
   const gchar *title;
   char *mbtitle;
@@ -509,12 +495,7 @@ gdk_window_new_internal (GdkWindow     *parent,
   g_return_val_if_fail (attributes != NULL, NULL);
 
   if (!parent)
-    {
-      screen = gdk_screen_get_default ();
-      parent = _gdk_root;
-    }
-  else
-    screen = gdk_drawable_get_screen (parent);
+    parent = _gdk_root;
 
   g_return_val_if_fail (GDK_IS_WINDOW (parent), NULL);
   
@@ -598,7 +579,7 @@ gdk_window_new_internal (GdkWindow     *parent,
         }
       else
 	{
-	  draw_impl->colormap = gdk_screen_get_system_colormap (screen);
+	  draw_impl->colormap = gdk_screen_get_system_colormap (_gdk_screen);
 	  g_object_ref (draw_impl->colormap);
 	}
     }
@@ -607,7 +588,7 @@ gdk_window_new_internal (GdkWindow     *parent,
       dwExStyle = WS_EX_TRANSPARENT;
       private->depth = 0;
       private->input_only = TRUE;
-      draw_impl->colormap = gdk_screen_get_system_colormap (screen);
+      draw_impl->colormap = gdk_screen_get_system_colormap (_gdk_screen);
       g_object_ref (draw_impl->colormap);
       GDK_NOTE (MISC, g_print ("... GDK_INPUT_ONLY, system colormap"));
     }
@@ -696,7 +677,7 @@ gdk_window_new_internal (GdkWindow     *parent,
   mbtitle = g_locale_from_utf8 (title, -1, NULL, NULL, NULL);
   
   hwndNew = CreateWindowEx (dwExStyle,
-			    MAKEINTRESOURCE(klass),
+			    MAKEINTRESOURCE (klass),
 			    mbtitle,
 			    dwStyle,
 			    ((attributes_mask & GDK_WA_X) ?
@@ -709,9 +690,9 @@ gdk_window_new_internal (GdkWindow     *parent,
 			    window);
   if (GDK_WINDOW_HWND (window) != hwndNew)
     {
-      g_warning("gdk_window_new: gdk_event_translate::WM_CREATE (%p, %p) HWND mismatch.",
-		GDK_WINDOW_HWND (window),
-		hwndNew);
+      g_warning ("gdk_window_new: gdk_event_translate::WM_CREATE (%p, %p) HWND mismatch.",
+		 GDK_WINDOW_HWND (window),
+		 hwndNew);
 
       /* HB: IHMO due to a race condition the handle was increased by
        * one, which causes much trouble. Because I can't find the 
@@ -781,7 +762,7 @@ gdk_window_foreign_new_for_display (GdkDisplay      *display,
   RECT rect;
   POINT point;
 
-  g_return_val_if_fail (display == gdk_display_get_default (), NULL);
+  g_return_val_if_fail (display == _gdk_display, NULL);
 
   window = g_object_new (GDK_TYPE_WINDOW, NULL);
   private = (GdkWindowObject *)window;
@@ -884,7 +865,6 @@ _gdk_windowing_window_destroy_foreign (GdkWindow *window)
 void
 gdk_window_destroy_notify (GdkWindow *window)
 {
-  g_return_if_fail (window != NULL);
   g_return_if_fail (GDK_IS_WINDOW (window));
 
   GDK_NOTE (EVENTS,
@@ -894,7 +874,7 @@ gdk_window_destroy_notify (GdkWindow *window)
 
   if (!GDK_WINDOW_DESTROYED (window))
     {
-      if (GDK_WINDOW_TYPE(window) != GDK_WINDOW_FOREIGN)
+      if (GDK_WINDOW_TYPE (window) != GDK_WINDOW_FOREIGN)
 	g_warning ("window %p unexpectedly destroyed",
 		   GDK_WINDOW_HWND (window));
 
@@ -911,16 +891,11 @@ get_outer_rect (GdkWindow *window,
 		gint       height,
 		RECT      *rect)
 {
-  LONG style, exstyle;
-      
-  style = GetWindowLong (GDK_WINDOW_HWND (window), GWL_STYLE);
-  exstyle = GetWindowLong (GDK_WINDOW_HWND (window), GWL_EXSTYLE);
-      
   rect->left = rect->top = 0;
   rect->right = width;
   rect->bottom = height;
       
-  API_CALL (AdjustWindowRectEx, (rect, style, FALSE, exstyle));
+  _gdk_win32_adjust_client_rect (window, rect);
 }
 
 static void
@@ -1123,7 +1098,7 @@ gdk_window_hide (GdkWindow *window)
 {
   GdkWindowObject *private;
   
-  g_return_if_fail (window != NULL);
+  g_return_if_fail (GDK_IS_WINDOW (window));
 
   private = (GdkWindowObject*) window;
   if (private->destroyed)
@@ -1160,7 +1135,7 @@ gdk_window_withdraw (GdkWindow *window)
 {
   GdkWindowObject *private;
   
-  g_return_if_fail (window != NULL);
+  g_return_if_fail (GDK_IS_WINDOW (window));
   
   private = (GdkWindowObject*) window;
   if (private->destroyed)
@@ -1181,7 +1156,6 @@ gdk_window_move (GdkWindow *window,
   GdkWindowObject *private = (GdkWindowObject *)window;
   GdkWindowImplWin32 *impl;
 
-  g_return_if_fail (window != NULL);
   g_return_if_fail (GDK_IS_WINDOW (window));
 
   if (GDK_WINDOW_DESTROYED (window))
@@ -1228,7 +1202,6 @@ gdk_window_resize (GdkWindow *window,
   GdkWindowObject *private = (GdkWindowObject*) window;
   GdkWindowImplWin32 *impl;
 
-  g_return_if_fail (window != NULL);
   g_return_if_fail (GDK_IS_WINDOW (window));
 
   if (GDK_WINDOW_DESTROYED (window))
@@ -1252,6 +1225,7 @@ gdk_window_resize (GdkWindow *window,
   else
     {
       RECT outer_rect;
+
       get_outer_rect (window, width, height, &outer_rect);
 
       GDK_NOTE (MISC, g_print ("... SetWindowPos(%p,NULL,0,0,%ld,%ld,"
@@ -1279,7 +1253,6 @@ gdk_window_move_resize (GdkWindow *window,
   GdkWindowObject *private = (GdkWindowObject*) window;
   GdkWindowImplWin32 *impl;
 
-  g_return_if_fail (window != NULL);
   g_return_if_fail (GDK_IS_WINDOW (window));
 
   if (GDK_WINDOW_DESTROYED (window))
@@ -1337,7 +1310,6 @@ gdk_window_reparent (GdkWindow *window,
   gboolean was_toplevel;
   LONG style;
 
-  g_return_if_fail (window != NULL);
   g_return_if_fail (GDK_IS_WINDOW (window));
   g_return_if_fail (new_parent == NULL || GDK_IS_WINDOW (new_parent));
   g_return_if_fail (window != _gdk_root);
@@ -1436,7 +1408,6 @@ _gdk_windowing_window_clear_area (GdkWindow *window,
 {
   GdkWindowImplWin32 *impl;
 
-  g_return_if_fail (window != NULL);
   g_return_if_fail (GDK_IS_WINDOW (window));
   
   impl = GDK_WINDOW_IMPL_WIN32 (GDK_WINDOW_OBJECT (window)->impl);
@@ -1467,7 +1438,6 @@ _gdk_windowing_window_clear_area_e (GdkWindow *window,
 				    gint       width,
 				    gint       height)
 {
-  g_return_if_fail (window != NULL);
   g_return_if_fail (GDK_IS_WINDOW (window));
   
   if (!GDK_WINDOW_DESTROYED (window))
@@ -1491,7 +1461,6 @@ _gdk_windowing_window_clear_area_e (GdkWindow *window,
 void
 gdk_window_raise (GdkWindow *window)
 {
-  g_return_if_fail (window != NULL);
   g_return_if_fail (GDK_IS_WINDOW (window));
   
   if (!GDK_WINDOW_DESTROYED (window))
@@ -1506,7 +1475,6 @@ gdk_window_raise (GdkWindow *window)
 void
 gdk_window_lower (GdkWindow *window)
 {
-  g_return_if_fail (window != NULL);
   g_return_if_fail (GDK_IS_WINDOW (window));
   
   if (!GDK_WINDOW_DESTROYED (window))
@@ -1537,7 +1505,6 @@ gdk_window_set_hints (GdkWindow *window,
 
   GdkWindowImplWin32 *impl;
 
-  g_return_if_fail (window != NULL);
   g_return_if_fail (GDK_IS_WINDOW (window));
   
   if (GDK_WINDOW_DESTROYED (window))
@@ -1600,13 +1567,7 @@ gdk_window_set_geometry_hints (GdkWindow      *window,
 			       GdkWindowHints  geom_mask)
 {
   GdkWindowImplWin32 *impl;
-#if 0
-  WINDOWPLACEMENT size_hints;
-  RECT rect;
-  gint new_width = 0, new_height = 0;
-#endif
 
-  g_return_if_fail (window != NULL);
   g_return_if_fail (GDK_IS_WINDOW (window));
   
   if (GDK_WINDOW_DESTROYED (window))
@@ -1627,92 +1588,39 @@ gdk_window_set_geometry_hints (GdkWindow      *window,
     {
       GDK_NOTE (MISC, g_print ("... MIN_SIZE: %dx%d\n",
 			       geometry->min_width, geometry->min_height));
-
-#if 0
-      /* Check if the current size of the window is in bounds */
-      GetClientRect (GDK_WINDOW_HWND (window), &rect);
-
-      if (rect.right < geometry->min_width &&
-	  rect.bottom < geometry->min_height)
-	{
-	  new_width = geometry->min_width;
-	  new_height = geometry->min_height;
-	}
-      else if (rect.right < geometry->min_width)
-	{
-	  new_width = geometry->min_width;
-	  new_height = rect.bottom;
-	}
-      else if (rect.bottom < geometry->min_height)
-	{
-	  new_width = rect.right;
-	  new_height = geometry->min_height;
-	}
-#endif
     }
   
   if (geom_mask & GDK_HINT_MAX_SIZE)
     {
       GDK_NOTE (MISC, g_print ("... MAX_SIZE: %dx%d\n",
 			       geometry->max_width, geometry->max_height));
-
-#if 0
-      /* Check if the current size of the window is in bounds */
-      GetClientRect (GDK_WINDOW_HWND (window), &rect);
-
-      if (rect.right > geometry->max_width &&
-	  rect.bottom > geometry->max_height)
-	{
-	  new_width = geometry->max_width;
-	  new_height = geometry->max_height;
-	}
-      else if (rect.right > geometry->max_width)
-	{
-	  new_width = geometry->max_width;
-	  new_height = rect.bottom;
-	}
-      else if (rect.bottom > geometry->max_height)
-	{
-	  new_width = rect.right;
-	  new_height = geometry->max_height;
-	}
-#endif
     }
 
-#if 0
-  /* Apply new size constraints */
-  if (new_width != 0 && new_height != 0)
-    gdk_window_resize (window, new_width, new_height);
-#endif
+  if ((geom_mask & GDK_HINT_MIN_SIZE) &&
+      (geom_mask & GDK_HINT_MAX_SIZE) &&
+      geometry->min_width == geometry->max_width &&
+      geometry->min_height == geometry->max_height)
+    gdk_window_set_decorations (window,
+				GDK_DECOR_ALL |
+				GDK_DECOR_RESIZEH |
+				GDK_DECOR_MAXIMIZE);
+  else if (geom_mask & GDK_HINT_MAX_SIZE)
+    {
+      gdk_window_set_decorations (window,
+				  GDK_DECOR_ALL |
+				  GDK_DECOR_MAXIMIZE);
+      gdk_window_set_decorations (window,
+				  GDK_DECOR_RESIZEH);
+    }
+  else
+    gdk_window_set_decorations (window,
+				GDK_DECOR_RESIZEH |
+				GDK_DECOR_MAXIMIZE);
 
   if (geom_mask & GDK_HINT_BASE_SIZE)
     {
       GDK_NOTE (MISC, g_print ("... BASE_SIZE: %dx%d\n",
 			       geometry->base_width, geometry->base_height));
-
-#if 0
-      size_hints.length = sizeof (size_hints);
-
-      if (API_CALL (GetWindowPlacement, (GDK_WINDOW_HWND (window), &size_hints)))
-	{
-	  GDK_NOTE (MISC,
-		    g_print ("... rcNormalPosition: (%ld,%ld)--(%ld,%ld)\n",
-			     size_hints.rcNormalPosition.left,
-			     size_hints.rcNormalPosition.top,
-			     size_hints.rcNormalPosition.right,
-			     size_hints.rcNormalPosition.bottom));
-	  size_hints.rcNormalPosition.right =
-	    size_hints.rcNormalPosition.left + geometry->base_width;
-	  size_hints.rcNormalPosition.bottom =
-	    size_hints.rcNormalPosition.top + geometry->base_height;
-	  GDK_NOTE (MISC, g_print ("... setting: rcNormal: (%ld,%ld)--(%ld,%ld)\n",
-				   size_hints.rcNormalPosition.left,
-				   size_hints.rcNormalPosition.top,
-				   size_hints.rcNormalPosition.right,
-				   size_hints.rcNormalPosition.bottom));
-	  API_CALL (SetWindowPlacement, (GDK_WINDOW_HWND (window), &size_hints));
-	}
-#endif
     }
   
   if (geom_mask & GDK_HINT_RESIZE_INC)
@@ -1768,7 +1676,6 @@ void
 gdk_window_set_role (GdkWindow   *window,
 		     const gchar *role)
 {
-  g_return_if_fail (window != NULL);
   g_return_if_fail (GDK_IS_WINDOW (window));
   
   GDK_NOTE (MISC, g_print ("gdk_window_set_role: %p: %s\n",
@@ -1783,7 +1690,6 @@ gdk_window_set_transient_for (GdkWindow *window,
 {
   HWND window_id, parent_id;
 
-  g_return_if_fail (window != NULL);
   g_return_if_fail (GDK_IS_WINDOW (window));
   
   GDK_NOTE (MISC, g_print ("gdk_window_set_transient_for: %p: %p\n",
@@ -1819,7 +1725,6 @@ gdk_window_set_background (GdkWindow      *window,
 {
   GdkWindowObject *private = (GdkWindowObject *)window;
   
-  g_return_if_fail (window != NULL);
   g_return_if_fail (GDK_IS_WINDOW (window));
   
   GDK_NOTE (MISC, g_print ("gdk_window_set_background: %p: %s\n",
@@ -1844,7 +1749,6 @@ gdk_window_set_back_pixmap (GdkWindow *window,
 {
   GdkWindowObject *private = (GdkWindowObject *)window;
 
-  g_return_if_fail (window != NULL);
   g_return_if_fail (GDK_IS_WINDOW (window));
   g_return_if_fail (pixmap == NULL || !parent_relative);
   g_return_if_fail (pixmap == NULL || gdk_drawable_get_depth (window) == gdk_drawable_get_depth (pixmap));
@@ -1889,7 +1793,6 @@ gdk_window_set_cursor (GdkWindow *window,
   HCURSOR hcursor;
   HCURSOR hprevcursor;
   
-  g_return_if_fail (window != NULL);
   g_return_if_fail (GDK_IS_WINDOW (window));
   
   impl = GDK_WINDOW_IMPL_WIN32 (GDK_WINDOW_OBJECT (window)->impl);
@@ -1961,7 +1864,7 @@ gdk_window_set_cursor (GdkWindow *window,
    */
   if (hprevcursor != NULL)
     {
-      if (GetCursor() == hprevcursor)
+      if (GetCursor () == hprevcursor)
 	{
 	  /* Look for a suitable cursor to use instead */
 	  hcursor = NULL;
@@ -2063,7 +1966,7 @@ gdk_window_get_origin (GdkWindow *window,
   gint tx = 0;
   gint ty = 0;
 
-  g_return_val_if_fail (window != NULL, 0);
+  g_return_val_if_fail (GDK_IS_WINDOW (window), 0);
 
   if (!GDK_WINDOW_DESTROYED (window))
     {
@@ -2238,11 +2141,10 @@ _gdk_windowing_get_pointer (GdkDisplay       *display,
 			    gint             *y,
 			    GdkModifierType  *mask)
 {
-  GdkScreen *default_screen = gdk_display_get_default_screen (display);
-  GdkWindow *root_window = gdk_screen_get_root_window (default_screen);
+  g_return_if_fail (display == _gdk_display);
   
-  *screen = default_screen;
-  _gdk_windowing_window_get_pointer (display, root_window, x, y, mask);
+  *screen = _gdk_screen;
+  _gdk_windowing_window_get_pointer (_gdk_display, _gdk_root, x, y, mask);
 }
 
 void
@@ -2251,7 +2153,10 @@ gdk_display_warp_pointer (GdkDisplay *display,
 			  gint        x,
 			  gint        y)
 {
-  SetCursorPos (x, y);
+  g_return_if_fail (display == _gdk_display);
+  g_return_if_fail (screen == _gdk_screen);
+
+  SetCursorPos (x - _gdk_offset_x, y - _gdk_offset_y);
 }
 
 GdkWindow*
@@ -2304,7 +2209,6 @@ _gdk_windowing_window_at_pointer (GdkDisplay *display,
 GdkEventMask  
 gdk_window_get_events (GdkWindow *window)
 {
-  g_return_val_if_fail (window != NULL, 0);
   g_return_val_if_fail (GDK_IS_WINDOW (window), 0);
 
   if (GDK_WINDOW_DESTROYED (window))
@@ -2317,7 +2221,6 @@ void
 gdk_window_set_events (GdkWindow   *window,
 		       GdkEventMask event_mask)
 {
-  g_return_if_fail (window != NULL);
   g_return_if_fail (GDK_IS_WINDOW (window));
 
   if (GDK_WINDOW_DESTROYED (window))
@@ -2359,7 +2262,8 @@ gdk_window_shape_combine_mask (GdkWindow *window,
 			       GDK_WINDOW_HWND (window),
 			       GDK_WINDOW_HWND (mask)));
 
-      _gdk_win32_get_adjusted_client_rect (window, &rect);
+      GetClientRect (GDK_WINDOW_HWND (window), &rect);
+      _gdk_win32_adjust_client_rect (window, &rect);
 
       OffsetRgn (hrgn, -rect.left, -rect.top);
       OffsetRgn (hrgn, x, y);
@@ -2382,7 +2286,6 @@ void
 gdk_window_set_override_redirect (GdkWindow *window,
 				  gboolean   override_redirect)
 {
-  g_return_if_fail (window != NULL);
   g_return_if_fail (GDK_IS_WINDOW (window));
 
   g_warning ("gdk_window_set_override_redirect not implemented");
@@ -2393,7 +2296,7 @@ gdk_window_set_accept_focus (GdkWindow *window,
 			     gboolean accept_focus)
 {
   GdkWindowObject *private;
-  g_return_if_fail (window != NULL);
+
   g_return_if_fail (GDK_IS_WINDOW (window));
 
   private = (GdkWindowObject *)window;  
@@ -2409,7 +2312,7 @@ gdk_window_set_focus_on_map (GdkWindow *window,
 			     gboolean focus_on_map)
 {
   GdkWindowObject *private;
-  g_return_if_fail (window != NULL);
+
   g_return_if_fail (GDK_IS_WINDOW (window));
 
   private = (GdkWindowObject *)window;  
@@ -2468,8 +2371,8 @@ gdk_window_set_icon_list (GdkWindow *window,
 	  big_i = i;
 	}
 
-      dw = ABS(w - small_w);
-      dh = ABS(h - small_h);
+      dw = ABS (w - small_w);
+      dh = ABS (h - small_h);
       diff = dw*dw + dh*dh;
       if (small_pixbuf == NULL || diff < small_diff)
 	{
@@ -2507,7 +2410,6 @@ gdk_window_set_icon (GdkWindow *window,
 		     GdkPixmap *pixmap,
 		     GdkBitmap *mask)
 {
-  g_return_if_fail (window != NULL);
   g_return_if_fail (GDK_IS_WINDOW (window));
 
   /* do nothing, use gdk_window_set_icon_list instead */
@@ -2517,13 +2419,22 @@ void
 gdk_window_set_icon_name (GdkWindow   *window, 
 			  const gchar *name)
 {
-  g_return_if_fail (window != NULL);
   g_return_if_fail (GDK_IS_WINDOW (window));
 
   if (GDK_WINDOW_DESTROYED (window))
     return;
   
+#if 0
+  /* This is not the correct thing to do. We should keep both the
+   * "normal" window title, and the icon name. When the window is
+   * minimized, call SetWindowText() with the icon name, and when the
+   * window is restored, with the normal window title. Also, the name
+   * is in UTF-8, so we should do the normal conversion to either wide
+   * chars or system codepage, and use either the W or A version of
+   * SetWindowText(), depending on Windows version.
+   */
   API_CALL (SetWindowText, (GDK_WINDOW_HWND (window), name));
+#endif
 }
 
 GdkWindow *
@@ -2544,10 +2455,9 @@ void
 gdk_window_set_group (GdkWindow *window, 
 		      GdkWindow *leader)
 {
-  g_return_if_fail (window != NULL);
   g_return_if_fail (GDK_IS_WINDOW (window));
-  g_return_if_fail (leader != NULL);
-  g_return_if_fail (GDK_IS_WINDOW (leader));
+  g_return_if_fail (GDK_WINDOW_TYPE (window) != GDK_WINDOW_CHILD);
+  g_return_if_fail (leader == NULL || GDK_IS_WINDOW (leader));
 
   if (GDK_WINDOW_DESTROYED (window) || GDK_WINDOW_DESTROYED (leader))
     return;
@@ -2555,19 +2465,54 @@ gdk_window_set_group (GdkWindow *window,
   g_warning ("gdk_window_set_group not implemented");
 }
 
+static void
+set_or_clear_style_bits (GdkWindow *window,
+			 gboolean   clear_bits,
+			 int        bits)
+{
+  LONG style, exstyle;
+  RECT rect, before, after;
+
+  style = GetWindowLong (GDK_WINDOW_HWND (window), GWL_STYLE);
+  exstyle = GetWindowLong (GDK_WINDOW_HWND (window), GWL_EXSTYLE);
+
+  GetClientRect (GDK_WINDOW_HWND (window), &before);
+  after = before;
+  AdjustWindowRectEx (&before, style, FALSE, exstyle);
+
+  if (clear_bits)
+    style &= ~bits;
+  else
+    style |= bits;
+
+  SetWindowLong (GDK_WINDOW_HWND (window), GWL_STYLE, style);
+
+  AdjustWindowRectEx (&after, style, FALSE, exstyle);
+
+  GetWindowRect (GDK_WINDOW_HWND (window), &rect);
+  rect.left += after.left - before.left;
+  rect.top += after.top - before.top;
+  rect.right += after.right - before.right;
+  rect.bottom += after.bottom - before.bottom;
+
+  SetWindowPos (GDK_WINDOW_HWND (window), NULL,
+		rect.left, rect.top,
+		rect.right - rect.left, rect.bottom - rect.top,
+		SWP_FRAMECHANGED | SWP_NOACTIVATE | 
+		SWP_NOREPOSITION | SWP_NOZORDER);
+}
+
 void
 gdk_window_set_decorations (GdkWindow      *window,
 			    GdkWMDecoration decorations)
 {
-  LONG style, bits;
-  const LONG settable_bits = WS_BORDER|WS_THICKFRAME|WS_CAPTION|WS_SYSMENU|WS_MINIMIZEBOX|WS_MAXIMIZEBOX;
+  int bits;
 
-  g_return_if_fail (window != NULL);
   g_return_if_fail (GDK_IS_WINDOW (window));
   
-  GDK_NOTE (MISC, g_print ("gdk_window_set_decorations: %p: %s%s%s%s%s%s%s\n",
+  GDK_NOTE (MISC, g_print ("gdk_window_set_decorations: %p: %s %s%s%s%s%s%s\n",
 			   GDK_WINDOW_HWND (window),
-			   (decorations & GDK_DECOR_ALL ? "ALL " : ""),
+			   (decorations & GDK_DECOR_ALL ? "clearing" : "setting"),
 			   (decorations & GDK_DECOR_BORDER ? "BORDER " : ""),
 			   (decorations & GDK_DECOR_RESIZEH ? "RESIZEH " : ""),
 			   (decorations & GDK_DECOR_TITLE ? "TITLE " : ""),
@@ -2575,8 +2520,6 @@ gdk_window_set_decorations (GdkWindow      *window,
 			   (decorations & GDK_DECOR_MINIMIZE ? "MINIMIZE " : ""),
 			   (decorations & GDK_DECOR_MAXIMIZE ? "MAXIMIZE " : "")));
 
-  style = GetWindowLong (GDK_WINDOW_HWND (window), GWL_STYLE);
-
   bits = 0;
 
   if (decorations & GDK_DECOR_BORDER)
@@ -2592,24 +2535,18 @@ gdk_window_set_decorations (GdkWindow      *window,
   if (decorations & GDK_DECOR_MAXIMIZE)
     bits |= WS_MAXIMIZEBOX;
 
-  if (decorations & GDK_DECOR_ALL)
-    style |= settable_bits, style &= ~bits;
-  else
-    style &= ~settable_bits, style |= bits;
-  
-  SetWindowLong (GDK_WINDOW_HWND (window), GWL_STYLE, style);
-  SetWindowPos (GDK_WINDOW_HWND (window), NULL,
-		0, 0, 0, 0,
-		SWP_FRAMECHANGED | SWP_NOACTIVATE | SWP_NOMOVE |
-		SWP_NOREPOSITION | SWP_NOSIZE | SWP_NOZORDER);
+  set_or_clear_style_bits (window, (decorations & GDK_DECOR_ALL), bits);
 }
 
 gboolean
-gdk_window_get_decorations(GdkWindow       *window,
-			   GdkWMDecoration *decorations)
+gdk_window_get_decorations (GdkWindow       *window,
+			    GdkWMDecoration *decorations)
 {
-  LONG style = GetWindowLong (GDK_WINDOW_HWND (window), GWL_STYLE);
+  LONG style;
 
+  g_return_val_if_fail (GDK_IS_WINDOW (window), FALSE);
+
+  style = GetWindowLong (GDK_WINDOW_HWND (window), GWL_STYLE);
   *decorations = 0;
 
   if (style & WS_BORDER)
@@ -2632,23 +2569,19 @@ void
 gdk_window_set_functions (GdkWindow    *window,
 			  GdkWMFunction functions)
 {
-  LONG style, bits;
-  const LONG settable_bits = (WS_THICKFRAME|WS_MINIMIZEBOX|WS_MAXIMIZEBOX|WS_SYSMENU);
+  int bits;
 
-  g_return_if_fail (window != NULL);
   g_return_if_fail (GDK_IS_WINDOW (window));
   
-  GDK_NOTE (MISC, g_print ("gdk_window_set_functions: %p: %s%s%s%s%s%s\n",
+  GDK_NOTE (MISC, g_print ("gdk_window_set_functions: %p: %s %s%s%s%s%s\n",
 			   GDK_WINDOW_HWND (window),
-			   (functions & GDK_FUNC_ALL ? "ALL " : ""),
+			   (functions & GDK_FUNC_ALL ? "clearing" : "setting"),
 			   (functions & GDK_FUNC_RESIZE ? "RESIZE " : ""),
 			   (functions & GDK_FUNC_MOVE ? "MOVE " : ""),
 			   (functions & GDK_FUNC_MINIMIZE ? "MINIMIZE " : ""),
 			   (functions & GDK_FUNC_MAXIMIZE ? "MAXIMIZE " : ""),
 			   (functions & GDK_FUNC_CLOSE ? "CLOSE " : "")));
 
-  style = GetWindowLong (GDK_WINDOW_HWND (window), GWL_STYLE);
-
   bits = 0;
 
   if (functions & GDK_FUNC_RESIZE)
@@ -2662,16 +2595,7 @@ gdk_window_set_functions (GdkWindow    *window,
   if (functions & GDK_FUNC_CLOSE)
     bits |= WS_SYSMENU;
   
-  if (functions & GDK_FUNC_ALL)
-    style |= settable_bits, style &= ~bits;
-  else
-    style &= ~settable_bits, style |= bits;
-  
-  SetWindowLong (GDK_WINDOW_HWND (window), GWL_STYLE, style);
-  SetWindowPos (GDK_WINDOW_HWND (window), NULL,
-		0, 0, 0, 0,
-		SWP_FRAMECHANGED | SWP_NOACTIVATE | SWP_NOMOVE |
-		SWP_NOREPOSITION | SWP_NOSIZE | SWP_NOZORDER);
+  set_or_clear_style_bits (window, (functions & GDK_FUNC_ALL), bits);
 }
 
 static void
@@ -2748,7 +2672,6 @@ gdk_propagate_shapes (HANDLE   win,
 void
 gdk_window_set_child_shapes (GdkWindow *window)
 {
-  g_return_if_fail (window != NULL);
   g_return_if_fail (GDK_IS_WINDOW (window));
    
   if (GDK_WINDOW_DESTROYED (window))
@@ -2760,7 +2683,6 @@ gdk_window_set_child_shapes (GdkWindow *window)
 void
 gdk_window_merge_child_shapes (GdkWindow *window)
 {
-  g_return_if_fail (window != NULL);
   g_return_if_fail (GDK_IS_WINDOW (window));
   
   if (GDK_WINDOW_DESTROYED (window))
@@ -2773,7 +2695,6 @@ gboolean
 gdk_window_set_static_gravities (GdkWindow *window,
 				 gboolean   use_static)
 {
-  g_return_val_if_fail (window != NULL, FALSE);
   g_return_val_if_fail (GDK_IS_WINDOW (window), FALSE);
 
   return !use_static;
@@ -2798,13 +2719,15 @@ gdk_window_begin_resize_drag (GdkWindow     *window,
    * the left pointer button was clicked in the suitable edge or corner. This
    * will only work if the button is down when this function is called, and
    * will only work with button 1 (left), since Windows only allows window
-   * dragging using the left mouse button */
+   * dragging using the left mouse button.
+   */
   if (button != 1)
     return;
   
-  /* Must break the automatic grab that occured when the button was pressed,
-   * otherwise it won't work */
-  gdk_display_pointer_ungrab (gdk_display_get_default (), 0);
+  /* Must break the automatic grab that occured when the button was
+   * pressed, otherwise it won't work.
+   */
+  gdk_display_pointer_ungrab (_gdk_display, 0);
 
   switch (edge)
     {
@@ -2843,7 +2766,7 @@ gdk_window_begin_resize_drag (GdkWindow     *window,
     }
 
   DefWindowProc (GDK_WINDOW_HWND (window), WM_NCLBUTTONDOWN, winedge,
-      MAKELPARAM (root_x - _gdk_offset_x, root_y - _gdk_offset_y));
+		 MAKELPARAM (root_x - _gdk_offset_x, root_y - _gdk_offset_y));
 }
 
 void
@@ -2862,16 +2785,18 @@ gdk_window_begin_move_drag (GdkWindow *window,
    * the left pointer button was clicked in the titlebar. This will only work
    * if the button is down when this function is called, and will only work
    * with button 1 (left), since Windows only allows window dragging using the
-   * left mouse button */
+   * left mouse button.
+   */
   if (button != 1)
     return;
   
   /* Must break the automatic grab that occured when the button was pressed,
-   * otherwise it won't work */
-  gdk_display_pointer_ungrab (gdk_display_get_default (), 0);
+   * otherwise it won't work.
+   */
+  gdk_display_pointer_ungrab (_gdk_display, 0);
 
   DefWindowProc (GDK_WINDOW_HWND (window), WM_NCLBUTTONDOWN, HTCAPTION,
-      MAKELPARAM (root_x - _gdk_offset_x, root_y - _gdk_offset_y));
+		 MAKELPARAM (root_x - _gdk_offset_x, root_y - _gdk_offset_y));
 }
 
 
@@ -2883,7 +2808,6 @@ gdk_window_iconify (GdkWindow *window)
 {
   HWND old_active_window;
 
-  g_return_if_fail (window != NULL);
   g_return_if_fail (GDK_IS_WINDOW (window));
 
   if (GDK_WINDOW_DESTROYED (window))
@@ -2911,7 +2835,6 @@ gdk_window_iconify (GdkWindow *window)
 void
 gdk_window_deiconify (GdkWindow *window)
 {
-  g_return_if_fail (window != NULL);
   g_return_if_fail (GDK_IS_WINDOW (window));
 
   if (GDK_WINDOW_DESTROYED (window))
@@ -3049,7 +2972,7 @@ gdk_window_unfullscreen (GdkWindow *window)
 
   g_return_if_fail (GDK_IS_WINDOW (window));
 
-  fi = g_object_get_data (G_OBJECT(window), "fullscreen-info");
+  fi = g_object_get_data (G_OBJECT (window), "fullscreen-info");
   if (fi)
     {
       GdkWindowImplWin32 *impl = GDK_WINDOW_IMPL_WIN32 (private->impl);
@@ -3136,7 +3059,6 @@ gdk_window_set_modal_hint (GdkWindow *window,
 {
   GdkWindowObject *private;
 
-  g_return_if_fail (window != NULL);
   g_return_if_fail (GDK_IS_WINDOW (window));
   
   if (GDK_WINDOW_DESTROYED (window))
@@ -3206,7 +3128,6 @@ void
 gdk_window_set_type_hint (GdkWindow        *window,
 			  GdkWindowTypeHint hint)
 {
-  g_return_if_fail (window != NULL);
   g_return_if_fail (GDK_IS_WINDOW (window));
   
   if (GDK_WINDOW_DESTROYED (window))
@@ -3267,7 +3188,7 @@ gdk_window_shape_combine_region (GdkWindow *window,
 GdkWindow *
 gdk_window_lookup_for_display (GdkDisplay *display, GdkNativeWindow anid)
 {
-  g_return_val_if_fail (display == gdk_display_get_default(), NULL);
+  g_return_val_if_fail (display == _gdk_display, NULL);
 
   return gdk_window_lookup (anid);
 }
