@@ -119,9 +119,9 @@ gtk_file_info_get_display_name (const GtkFileInfo *info)
  * gtk_file_info_get_display_key:
  * @info: a #GtkFileInfo
  * 
- * Returns results of g_utf8_collate_key() on the display name
- * for @info. This is useful when sorting a bunch of #GtkFileInfo
- * structures since the collate key will be only computed once.
+ * Returns for the collation key for the display name for @info. 
+ * This is useful when sorting a bunch of #GtkFileInfo structures 
+ * since the collate key will be only computed once.
  * 
  * Return value: The collate key for the display name, or %NULL
  *   if the display name hasn't been set.
@@ -135,7 +135,7 @@ gtk_file_info_get_display_key (const GtkFileInfo *info)
     {
       /* Since info->display_key is only a cache, we cast off the const
        */
-      ((GtkFileInfo *)info)->display_key = g_utf8_collate_key (info->display_name, -1);
+      ((GtkFileInfo *)info)->display_key = g_utf8_collate_key_for_filename (info->display_name, -1);
     }
 	
   return info->display_key;
@@ -772,6 +772,59 @@ gtk_file_system_list_bookmarks (GtkFileSystem *file_system)
   return GTK_FILE_SYSTEM_GET_IFACE (file_system)->list_bookmarks (file_system);
 }
 
+/**
+ * gtk_file_system_get_bookmark_label:
+ * @file_system: a #GtkFileSystem
+ * @path: path of the bookmark 
+ *
+ * Gets the label to display for a bookmark, or %NULL.
+ *
+ * Returns: the label for the bookmark @path
+ *
+ * Since: 2.8
+ */
+gchar *
+gtk_file_system_get_bookmark_label (GtkFileSystem     *file_system,
+				    const GtkFilePath *path)
+{
+  GtkFileSystemIface *iface;
+
+  g_return_val_if_fail (GTK_IS_FILE_SYSTEM (file_system), NULL);
+  g_return_val_if_fail (path != NULL, FALSE);
+
+  iface = GTK_FILE_SYSTEM_GET_IFACE (file_system);
+  if (iface->get_bookmark_label)
+    return iface->get_bookmark_label (file_system, path);
+
+  return NULL;
+}
+
+/**
+ * gtk_file_system_set_bookmark_label:
+ * @file_system: a #GtkFileSystem
+ * @path: path of the bookmark 
+ * @label: the label for the bookmark, or %NULL to display
+ *   the path itself
+ *
+ * Sets the label to display for a bookmark.
+ *
+ * Since: 2.8
+ */
+void
+gtk_file_system_set_bookmark_label (GtkFileSystem     *file_system,
+				    const GtkFilePath *path,
+				    const gchar       *label)
+{
+  GtkFileSystemIface *iface;
+
+  g_return_if_fail (GTK_IS_FILE_SYSTEM (file_system));
+  g_return_if_fail (path != NULL);
+
+  iface = GTK_FILE_SYSTEM_GET_IFACE (file_system);
+  if (iface->set_bookmark_label)
+    iface->set_bookmark_label (file_system, path, label);
+}
+
 /*****************************************
  *             GtkFileFolder             *
  *****************************************/
@@ -887,12 +940,15 @@ gtk_file_folder_get_info (GtkFileFolder     *folder,
 gboolean
 gtk_file_folder_is_finished_loading (GtkFileFolder *folder)
 {
+  GtkFileFolderIface *iface;
+
   g_return_val_if_fail (GTK_IS_FILE_FOLDER (folder), TRUE);
 
-  if (!GTK_FILE_FOLDER_GET_IFACE (folder)->is_finished_loading)
+  iface = GTK_FILE_FOLDER_GET_IFACE (folder);
+  if (!iface->is_finished_loading)
     return TRUE;
   else
-    return GTK_FILE_FOLDER_GET_IFACE (folder)->is_finished_loading (folder);
+    return iface->is_finished_loading (folder);
 }
 
 
@@ -1018,7 +1074,7 @@ gtk_file_system_module_load (GTypeModule *module)
 {
   GtkFileSystemModule *fs_module = GTK_FILE_SYSTEM_MODULE (module);
   
-  fs_module->library = g_module_open (fs_module->path, 0);
+  fs_module->library = g_module_open (fs_module->path, G_MODULE_BIND_LAZY);
   if (!fs_module->library)
     {
       g_warning (g_module_error());
