@@ -1,4 +1,4 @@
-/* gtkpathbar.h
+/* gtkpathbar.c
  * Copyright (C) 2004  Red Hat, Inc.,  Jonathan Blandford <jrb@gnome.org>
  *
  * This library is free software; you can redistribute it and/or
@@ -19,11 +19,11 @@
 
 #include <config.h>
 #include <string.h>
-#include "gtkalias.h"
 #include "gtkpathbar.h"
 #include "gtktogglebutton.h"
 #include "gtkalignment.h"
 #include "gtkarrow.h"
+#include "gtkdnd.h"
 #include "gtkimage.h"
 #include "gtkintl.h"
 #include "gtkicontheme.h"
@@ -32,6 +32,7 @@
 #include "gtkhbox.h"
 #include "gtkmain.h"
 #include "gtkmarshalers.h"
+#include "gtkalias.h"
 
 enum {
   PATH_CLICKED,
@@ -47,10 +48,13 @@ typedef enum {
 
 #define BUTTON_DATA(x) ((ButtonData *)(x))
 
+#define SCROLL_TIMEOUT           150
+#define INITIAL_SCROLL_TIMEOUT   300
+
 static guint path_bar_signals [LAST_SIGNAL] = { 0 };
 
 /* Icon size for if we can't get it from the theme */
-#define FALLBACK_ICON_SIZE 20
+#define FALLBACK_ICON_SIZE 16
 
 typedef struct _ButtonData ButtonData;
 
@@ -62,9 +66,14 @@ struct _ButtonData
   GtkFilePath *path;
   GtkWidget *image;
   GtkWidget *label;
-  gboolean ignore_changes;
-  gboolean file_is_hidden;
+  guint ignore_changes : 1;
+  guint file_is_hidden : 1;
 };
+/* This macro is used to check if a button can be used as a fake root.
+ * All buttons in front of a fake root are automatically hidden when in a
+ * directory below a fake root and replaced with the "<" arrow button.
+ */
+#define BUTTON_IS_FAKE_ROOT(button) ((button)->type == HOME_BUTTON)
 
 G_DEFINE_TYPE (GtkPathBar,
 	       gtk_path_bar,
@@ -74,6 +83,7 @@ static void gtk_path_bar_finalize                 (GObject          *object);
 static void gtk_path_bar_dispose                  (GObject          *object);
 static void gtk_path_bar_size_request             (GtkWidget        *widget,
 						   GtkRequisition   *requisition);
+static void gtk_path_bar_unmap                    (GtkWidget          *widget);
 static void gtk_path_bar_size_allocate            (GtkWidget        *widget,
 						   GtkAllocation    *allocation);
 static void gtk_path_bar_add                      (GtkContainer     *container,
@@ -88,6 +98,17 @@ static void gtk_path_bar_scroll_up                (GtkWidget        *button,
 						   GtkPathBar       *path_bar);
 static void gtk_path_bar_scroll_down              (GtkWidget        *button,
 						   GtkPathBar       *path_bar);
+static void gtk_path_bar_stop_scrolling           (GtkPathBar       *path_bar);
+static gboolean gtk_path_bar_slider_button_press  (GtkWidget        *widget,
+						   GdkEventButton   *event,
+						   GtkPathBar       *path_bar);
+static gboolean gtk_path_bar_slider_button_release(GtkWidget        *widget,
+						   GdkEventButton   *event,
+						   GtkPathBar       *path_bar);
+static void gtk_path_bar_grab_notify              (GtkWidget        *widget,
+						   gboolean          was_grabbed);
+static void gtk_path_bar_state_changed            (GtkWidget        *widget,
+						   GtkStateType      previous_state);
 static void gtk_path_bar_style_set                (GtkWidget        *widget,
 						   GtkStyle         *previous_style);
 static void gtk_path_bar_screen_changed           (GtkWidget        *widget,
@@ -106,6 +127,7 @@ get_slider_button (GtkPathBar  *path_bar,
   gtk_widget_push_composite_child ();
 
   button = gtk_button_new ();
+  gtk_button_set_focus_on_click (button, FALSE);
   gtk_container_add (GTK_CONTAINER (button), gtk_arrow_new (arrow_type, GTK_SHADOW_OUT));
   gtk_container_add (GTK_CONTAINER (path_bar), button);
   gtk_widget_show_all (button);
@@ -125,9 +147,14 @@ gtk_path_bar_init (GtkPathBar *path_bar)
   path_bar->up_slider_button = get_slider_button (path_bar, GTK_ARROW_LEFT);
   path_bar->down_slider_button = get_slider_button (path_bar, GTK_ARROW_RIGHT);
   path_bar->icon_size = FALLBACK_ICON_SIZE;
-
+  
   g_signal_connect (path_bar->up_slider_button, "clicked", G_CALLBACK (gtk_path_bar_scroll_up), path_bar);
   g_signal_connect (path_bar->down_slider_button, "clicked", G_CALLBACK (gtk_path_bar_scroll_down), path_bar);
+
+  g_signal_connect (path_bar->up_slider_button, "button_press_event", G_CALLBACK (gtk_path_bar_slider_button_press), path_bar);
+  g_signal_connect (path_bar->up_slider_button, "button_release_event", G_CALLBACK (gtk_path_bar_slider_button_release), path_bar);
+  g_signal_connect (path_bar->down_slider_button, "button_press_event", G_CALLBACK (gtk_path_bar_slider_button_press), path_bar);
+  g_signal_connect (path_bar->down_slider_button, "button_release_event", G_CALLBACK (gtk_path_bar_slider_button_release), path_bar);
 }
 
 static void
@@ -147,9 +174,12 @@ gtk_path_bar_class_init (GtkPathBarClass *path_bar_class)
   gobject_class->dispose = gtk_path_bar_dispose;
 
   widget_class->size_request = gtk_path_bar_size_request;
+  widget_class->unmap = gtk_path_bar_unmap;
   widget_class->size_allocate = gtk_path_bar_size_allocate;
   widget_class->style_set = gtk_path_bar_style_set;
   widget_class->screen_changed = gtk_path_bar_screen_changed;
+  widget_class->grab_notify = gtk_path_bar_grab_notify;
+  widget_class->state_changed = gtk_path_bar_state_changed;
 
   container_class->add = gtk_path_bar_add;
   container_class->forall = gtk_path_bar_forall;
@@ -163,8 +193,9 @@ gtk_path_bar_class_init (GtkPathBarClass *path_bar_class)
 		  G_SIGNAL_RUN_FIRST,
 		  G_STRUCT_OFFSET (GtkPathBarClass, path_clicked),
 		  NULL, NULL,
-		  _gtk_marshal_VOID__POINTER_BOOLEAN,
-		  G_TYPE_NONE, 2,
+		  _gtk_marshal_VOID__POINTER_POINTER_BOOLEAN,
+		  G_TYPE_NONE, 3,
+		  G_TYPE_POINTER,
 		  G_TYPE_POINTER,
 		  G_TYPE_BOOLEAN);
 }
@@ -176,6 +207,9 @@ gtk_path_bar_finalize (GObject *object)
   GtkPathBar *path_bar;
 
   path_bar = GTK_PATH_BAR (object);
+
+  gtk_path_bar_stop_scrolling (path_bar);
+
   g_list_free (path_bar->button_list);
   if (path_bar->root_path)
     gtk_file_path_free (path_bar->root_path);
@@ -253,7 +287,7 @@ gtk_path_bar_size_request (GtkWidget      *widget,
   /* Theoretically, the slider could be bigger than the other button.  But we're
    * not going to worry about that now.
    */
-  path_bar->slider_width = requisition->height / 2 + 5;
+  path_bar->slider_width = MIN(requisition->height * 2 / 3 + 5, requisition->height);
   if (path_bar->button_list && path_bar->button_list->next != NULL)
     requisition->width += (path_bar->spacing + path_bar->slider_width) * 2;
 
@@ -287,6 +321,14 @@ gtk_path_bar_update_slider_buttons (GtkPathBar *path_bar)
     }
 }
 
+static void
+gtk_path_bar_unmap (GtkWidget *widget)
+{
+  gtk_path_bar_stop_scrolling (GTK_PATH_BAR (widget));
+
+  GTK_WIDGET_CLASS (gtk_path_bar_parent_class)->unmap (widget);
+}
+
 /* This is a tad complicated
  */
 static void
@@ -316,17 +358,26 @@ gtk_path_bar_size_allocate (GtkWidget     *widget,
   allocation_width = allocation->width - 2 * border_width;
 
   /* First, we check to see if we need the scrollbars. */
-  width = BUTTON_DATA (path_bar->button_list->data)->button->requisition.width;
-  for (list = path_bar->button_list->next; list; list = list->next)
+  if (path_bar->fake_root)
+    width = path_bar->spacing + path_bar->slider_width;
+  else
+      width = 0;
+
+  for (list = path_bar->button_list; list; list = list->next)
     {
       child = BUTTON_DATA (list->data)->button;
 
       width += child->requisition.width + path_bar->spacing;
+      if (list == path_bar->fake_root)
+	break;
     }
 
   if (width <= allocation_width)
     {
-      first_button = g_list_last (path_bar->button_list);
+      if (path_bar->fake_root)
+	first_button = path_bar->fake_root;
+      else
+	first_button = g_list_last (path_bar->button_list);
     }
   else
     {
@@ -353,6 +404,8 @@ gtk_path_bar_size_allocate (GtkWidget     *widget,
 	  if (width + child->requisition.width +
 	      path_bar->spacing + slider_space > allocation_width)
 	    reached_end = TRUE;
+	  else if (list == path_bar->fake_root)
+	    break;
 	  else
 	    width += child->requisition.width + path_bar->spacing;
 
@@ -361,7 +414,7 @@ gtk_path_bar_size_allocate (GtkWidget     *widget,
 
       /* Finally, we walk up, seeing how many of the previous buttons we can
        * add */
-      while (first_button->next && ! reached_end)
+      while (first_button->next && !reached_end)
 	{
 	  child = BUTTON_DATA (first_button->next->data)->button;
 
@@ -372,6 +425,8 @@ gtk_path_bar_size_allocate (GtkWidget     *widget,
 	  else
 	    {
 	      width += child->requisition.width + path_bar->spacing;
+	      if (first_button == path_bar->fake_root)
+		break;
 	      first_button = first_button->next;
 	    }
 	}
@@ -384,7 +439,7 @@ gtk_path_bar_size_allocate (GtkWidget     *widget,
   if (direction == GTK_TEXT_DIR_RTL)
     {
       child_allocation.x = allocation->x + allocation->width - border_width;
-      if (need_sliders)
+      if (need_sliders || path_bar->fake_root)
 	{
 	  child_allocation.x -= (path_bar->spacing + path_bar->slider_width);
 	  up_slider_offset = allocation->width - border_width - path_bar->slider_width;
@@ -393,7 +448,7 @@ gtk_path_bar_size_allocate (GtkWidget     *widget,
   else
     {
       child_allocation.x = allocation->x + border_width;
-      if (need_sliders)
+      if (need_sliders || path_bar->fake_root)
 	{
 	  up_slider_offset = border_width;
 	  child_allocation.x += (path_bar->spacing + path_bar->slider_width);
@@ -428,11 +483,13 @@ gtk_path_bar_size_allocate (GtkWidget     *widget,
 	{
 	  child_allocation.x -= path_bar->spacing;
 	  down_slider_offset = child_allocation.x - widget->allocation.x - path_bar->slider_width;
+	  down_slider_offset = border_width;
 	}
       else
 	{
-	  child_allocation.x += child_allocation.width + path_bar->spacing;
 	  down_slider_offset = child_allocation.x - widget->allocation.x;
+	  down_slider_offset = allocation->width - border_width - path_bar->slider_width;
+	  child_allocation.x += child_allocation.width + path_bar->spacing;
 	}
     }
   /* Now we go hide all the widgets that don't fit */
@@ -446,27 +503,30 @@ gtk_path_bar_size_allocate (GtkWidget     *widget,
       gtk_widget_set_child_visible (BUTTON_DATA (list->data)->button, FALSE);
     }
 
-  if (need_sliders)
+  if (need_sliders || path_bar->fake_root)
     {
       child_allocation.width = path_bar->slider_width;
-      
       child_allocation.x = up_slider_offset + allocation->x;
       gtk_widget_size_allocate (path_bar->up_slider_button, &child_allocation);
 
+      gtk_widget_set_child_visible (path_bar->up_slider_button, TRUE);
+      gtk_widget_show_all (path_bar->up_slider_button);
+    }
+  else
+    gtk_widget_set_child_visible (path_bar->up_slider_button, FALSE);
+      
+  if (need_sliders)
+    {
+      child_allocation.width = path_bar->slider_width;
       child_allocation.x = down_slider_offset + allocation->x;
       gtk_widget_size_allocate (path_bar->down_slider_button, &child_allocation);
 
-      gtk_widget_set_child_visible (path_bar->up_slider_button, TRUE);
       gtk_widget_set_child_visible (path_bar->down_slider_button, TRUE);
-      gtk_widget_show_all (path_bar->up_slider_button);
       gtk_widget_show_all (path_bar->down_slider_button);
       gtk_path_bar_update_slider_buttons (path_bar);
     }
   else
-    {
-      gtk_widget_set_child_visible (path_bar->up_slider_button, FALSE);
-      gtk_widget_set_child_visible (path_bar->down_slider_button, FALSE);
-    }
+    gtk_widget_set_child_visible (path_bar->down_slider_button, FALSE);
 }
 
 static void
@@ -587,7 +647,13 @@ gtk_path_bar_scroll_down (GtkWidget *button, GtkPathBar *path_bar)
   gint space_needed;
   gint border_width;
   GtkTextDirection direction;
-  
+
+  if (path_bar->ignore_click)
+    {
+      path_bar->ignore_click = FALSE;
+      return;   
+    }
+
   gtk_widget_queue_resize (GTK_WIDGET (path_bar));
 
   border_width = GTK_CONTAINER (path_bar)->border_width;
@@ -639,18 +705,133 @@ gtk_path_bar_scroll_up (GtkWidget *button, GtkPathBar *path_bar)
 {
   GList *list;
 
+  if (path_bar->ignore_click)
+    {
+      path_bar->ignore_click = FALSE;
+      return;   
+    }
+
   gtk_widget_queue_resize (GTK_WIDGET (path_bar));
 
   for (list = g_list_last (path_bar->button_list); list; list = list->prev)
     {
       if (list->prev && gtk_widget_get_child_visible (BUTTON_DATA (list->prev->data)->button))
 	{
+	  if (list->prev == path_bar->fake_root)
+	    path_bar->fake_root = NULL;
 	  path_bar->first_scrolled_button = list;
 	  return;
 	}
     }
 }
 
+static gboolean
+gtk_path_bar_scroll_timeout (GtkPathBar *path_bar)
+{
+  gboolean retval = FALSE;
+
+  GDK_THREADS_ENTER ();
+
+  if (path_bar->timer)
+    {
+      if (path_bar->scrolling_up)
+	gtk_path_bar_scroll_up (path_bar->up_slider_button, path_bar);
+      else 
+	gtk_path_bar_scroll_down (path_bar->down_slider_button, path_bar);
+
+      if (path_bar->need_timer) 
+	{
+	  path_bar->need_timer = FALSE;
+
+	  path_bar->timer = g_timeout_add (SCROLL_TIMEOUT,
+					   (GSourceFunc)gtk_path_bar_scroll_timeout,
+					   path_bar);
+	  
+	}
+      else
+	retval = TRUE;
+      
+    }
+
+  GDK_THREADS_LEAVE ();
+
+  return retval;
+}
+
+static void 
+gtk_path_bar_stop_scrolling (GtkPathBar *path_bar)
+{
+  if (path_bar->timer)
+    {
+      g_source_remove (path_bar->timer);
+      path_bar->timer = 0;
+      path_bar->need_timer = FALSE;
+    }
+}
+
+static gboolean
+gtk_path_bar_slider_button_press (GtkWidget      *widget, 
+				  GdkEventButton *event,
+				  GtkPathBar     *path_bar)
+{
+  if (event->type != GDK_BUTTON_PRESS || event->button != 1)
+    return FALSE;
+
+  path_bar->ignore_click = FALSE;
+
+  if (widget == path_bar->up_slider_button)
+    {
+      path_bar->scrolling_up = TRUE;
+      gtk_path_bar_scroll_up (path_bar->up_slider_button, path_bar);
+    }
+  else if (widget == path_bar->down_slider_button) 
+    {
+      path_bar->scrolling_up = FALSE;
+      gtk_path_bar_scroll_down (path_bar->down_slider_button, path_bar);
+    }
+
+  if (!path_bar->timer)
+    {
+      path_bar->need_timer = TRUE;
+      path_bar->timer = g_timeout_add (INITIAL_SCROLL_TIMEOUT,
+				       (GSourceFunc)gtk_path_bar_scroll_timeout,
+				       path_bar);
+    }
+
+  return FALSE;
+}
+
+static gboolean
+gtk_path_bar_slider_button_release (GtkWidget      *widget, 
+				    GdkEventButton *event,
+				    GtkPathBar     *path_bar)
+{
+  if (event->type != GDK_BUTTON_RELEASE)
+    return FALSE;
+
+  path_bar->ignore_click = TRUE;
+  gtk_path_bar_stop_scrolling (path_bar);
+
+  return FALSE;
+}
+
+static void
+gtk_path_bar_grab_notify (GtkWidget *widget,
+			  gboolean   was_grabbed)
+{
+  if (!was_grabbed)
+    gtk_path_bar_stop_scrolling (GTK_PATH_BAR (widget));
+}
+
+static void
+gtk_path_bar_state_changed (GtkWidget    *widget,
+			    GtkStateType  previous_state)
+{
+  if (!GTK_WIDGET_IS_SENSITIVE (widget)) 
+    gtk_path_bar_stop_scrolling (GTK_PATH_BAR (widget));
+}
+
+
 /* Changes the icons wherever it is needed */
 static void
 reload_icons (GtkPathBar *path_bar)
@@ -696,7 +877,7 @@ change_icon_theme (GtkPathBar *path_bar)
 
   settings = gtk_settings_get_for_screen (gtk_widget_get_screen (GTK_WIDGET (path_bar)));
 
-  if (gtk_icon_size_lookup_for_settings (settings, GTK_ICON_SIZE_BUTTON, &width, &height))
+  if (gtk_icon_size_lookup_for_settings (settings, GTK_ICON_SIZE_MENU, &width, &height))
     path_bar->icon_size = MAX (width, height);
   else
     path_bar->icon_size = FALLBACK_ICON_SIZE;
@@ -741,6 +922,7 @@ gtk_path_bar_clear_buttons (GtkPathBar *path_bar)
       gtk_container_remove (GTK_CONTAINER (path_bar), BUTTON_DATA (path_bar->button_list->data)->button);
     }
   path_bar->first_scrolled_button = NULL;
+  path_bar->fake_root = NULL;
 }
 
 static void
@@ -763,17 +945,23 @@ button_clicked_cb (GtkWidget *button,
 
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
 
+  GtkFilePath *child_path;
   if (button_list->prev)
     {
       ButtonData *child_data;
 
       child_data = BUTTON_DATA (button_list->prev->data);
+      child_path = child_data->path;
       child_is_hidden = child_data->file_is_hidden;
     }
   else
-    child_is_hidden = FALSE;
+    {
+      child_path = NULL;
+      child_is_hidden = FALSE;
+    }
 
-  g_signal_emit (path_bar, path_bar_signals [PATH_CLICKED], 0, button_data->path, child_is_hidden);
+  g_signal_emit (path_bar, path_bar_signals [PATH_CLICKED], 0,
+		 button_data->path, child_path, child_is_hidden);
 }
 
 static GdkPixbuf *
@@ -839,13 +1027,7 @@ button_data_free (ButtonData *button_data)
 static const char *
 get_dir_name (ButtonData *button_data)
 {
-  if (button_data->type == HOME_BUTTON)
-    return _("Home");
-  else if (button_data->type == DESKTOP_BUTTON)
-    return _("Desktop");
-  else
-    return button_data->dir_name;
-
+  return button_data->dir_name;
 }
 
 /* We always want to request the same size for the label, whether
@@ -929,6 +1111,34 @@ find_button_type (GtkPathBar  *path_bar,
  return NORMAL_BUTTON;
 }
 
+static void
+button_drag_data_get_cb (GtkWidget          *widget,
+			 GdkDragContext     *context,
+			 GtkSelectionData   *selection_data,
+			 guint               info,
+			 guint               time_,
+			 gpointer            data)
+{
+  ButtonData *button_data;
+  GtkPathBar *path_bar;
+  char *uri;
+  char *uri_list;
+
+  button_data = data;
+  path_bar = GTK_PATH_BAR (widget->parent); /* the button's parent *is* the path bar */
+
+  uri = gtk_file_system_path_to_uri (path_bar->file_system, button_data->path);
+  uri_list = g_strconcat (uri, "\r\n", NULL);
+  g_free (uri);
+
+  gtk_selection_data_set (selection_data,
+			  selection_data->target,
+			  8,
+			  uri_list,
+			  strlen (uri_list));
+  g_free (uri_list);
+}
+
 static ButtonData *
 make_directory_button (GtkPathBar  *path_bar,
 		       const char  *dir_name,
@@ -936,15 +1146,21 @@ make_directory_button (GtkPathBar  *path_bar,
 		       gboolean     current_dir,
 		       gboolean     file_is_hidden)
 {
+  const GtkTargetEntry targets[] = {
+    { "text/uri-list", 0, 0 }
+  };
+
   GtkWidget *child = NULL;
   GtkWidget *label_alignment = NULL;
   ButtonData *button_data;
 
+  file_is_hidden = !! file_is_hidden;
   /* Is it a special button? */
   button_data = g_new0 (ButtonData, 1);
 
   button_data->type = find_button_type (path_bar, path);
   button_data->button = gtk_toggle_button_new ();
+  gtk_button_set_focus_on_click (button_data->button, FALSE);
 
   switch (button_data->type)
     {
@@ -957,7 +1173,7 @@ make_directory_button (GtkPathBar  *path_bar,
     case DESKTOP_BUTTON:
       button_data->image = gtk_image_new ();
       button_data->label = gtk_label_new (NULL);
-      label_alignment = gtk_alignment_new (0.5, 0.5, 0., 0.);
+      label_alignment = gtk_alignment_new (0.5, 0.5, 1.0, 1.0);
       gtk_container_add (GTK_CONTAINER (label_alignment), button_data->label);
       child = gtk_hbox_new (FALSE, 2);
       gtk_box_pack_start (GTK_BOX (child), button_data->image, FALSE, FALSE, 0);
@@ -966,7 +1182,7 @@ make_directory_button (GtkPathBar  *path_bar,
     case NORMAL_BUTTON:
     default:
       button_data->label = gtk_label_new (NULL);
-      label_alignment = gtk_alignment_new (0.5, 0.5, 0., 0.);
+      label_alignment = gtk_alignment_new (0.5, 0.5, 1.0, 1.0);
       gtk_container_add (GTK_CONTAINER (label_alignment), button_data->label);
       child = label_alignment;
       button_data->image = NULL;
@@ -995,6 +1211,14 @@ make_directory_button (GtkPathBar  *path_bar,
   g_object_weak_ref (G_OBJECT (button_data->button),
 		     (GWeakNotify) button_data_free, button_data);
 
+  gtk_drag_source_set (button_data->button,
+		       GDK_BUTTON1_MASK,
+		       targets,
+		       G_N_ELEMENTS (targets),
+		       GDK_ACTION_COPY);
+  g_signal_connect (button_data->button, "drag-data-get",
+		    G_CALLBACK (button_drag_data_get_cb), button_data);
+
   return button_data;
 }
 
@@ -1005,6 +1229,7 @@ gtk_path_bar_check_parent_path (GtkPathBar         *path_bar,
 {
   GList *list;
   GList *current_path = NULL;
+  gboolean need_new_fake_root = FALSE;
 
   for (list = path_bar->button_list; list; list = list->next)
     {
@@ -1016,16 +1241,41 @@ gtk_path_bar_check_parent_path (GtkPathBar         *path_bar,
 	  current_path = list;
 	  break;
 	}
+      if (list == path_bar->fake_root)
+	need_new_fake_root = TRUE;
     }
 
   if (current_path)
     {
+      if (need_new_fake_root)
+	{
+	  path_bar->fake_root = NULL;
+	  for (list = current_path; list; list = list->next)
+	    {
+	      ButtonData *button_data;
+
+	      button_data = list->data;
+	      if (BUTTON_IS_FAKE_ROOT (button_data))
+		{
+		  path_bar->fake_root = list;
+		  break;
+		}
+	    }
+	}
+
       for (list = path_bar->button_list; list; list = list->next)
 	{
 	  gtk_path_bar_update_button_appearance (path_bar,
 						 BUTTON_DATA (list->data),
 						 (list == current_path) ? TRUE : FALSE);
 	}
+
+      if (!gtk_widget_get_child_visible (BUTTON_DATA (current_path->data)->button))
+	{
+	  path_bar->first_scrolled_button = current_path;
+	  gtk_widget_queue_resize (GTK_WIDGET (path_bar));
+	}
+
       return TRUE;
     }
   return FALSE;
@@ -1034,12 +1284,14 @@ gtk_path_bar_check_parent_path (GtkPathBar         *path_bar,
 gboolean
 _gtk_path_bar_set_path (GtkPathBar         *path_bar,
 			const GtkFilePath  *file_path,
+			const gboolean      keep_trail,     
 			GError            **error)
 {
   GtkFilePath *path;
   gboolean first_directory = TRUE;
   gboolean result;
   GList *new_buttons = NULL;
+  GList *fake_root = NULL;
 
   g_return_val_if_fail (GTK_IS_PATH_BAR (path_bar), FALSE);
   g_return_val_if_fail (file_path != NULL, FALSE);
@@ -1049,7 +1301,8 @@ _gtk_path_bar_set_path (GtkPathBar         *path_bar,
   /* Check whether the new path is already present in the pathbar as buttons.
    * This could be a parent directory or a previous selected subdirectory.
    */
-  if (gtk_path_bar_check_parent_path (path_bar, file_path, path_bar->file_system))
+  if (keep_trail &&
+      gtk_path_bar_check_parent_path (path_bar, file_path, path_bar->file_system))
     return TRUE;
 
   path = gtk_file_path_copy (file_path);
@@ -1107,16 +1360,10 @@ _gtk_path_bar_set_path (GtkPathBar         *path_bar,
       gtk_file_info_free (file_info);
       gtk_file_path_free (path);
 
-      g_object_ref (button_data->button);
-      gtk_object_sink (GTK_OBJECT (button_data->button));
-
       new_buttons = g_list_prepend (new_buttons, button_data);
 
-      if (button_data->type != NORMAL_BUTTON)
-	{
-	  gtk_file_path_free (parent_path);
-	  break;
-	}
+      if (BUTTON_IS_FAKE_ROOT (button_data))
+	fake_root = new_buttons;
 
       path = parent_path;
       first_directory = FALSE;
@@ -1128,6 +1375,7 @@ _gtk_path_bar_set_path (GtkPathBar         *path_bar,
 
       gtk_path_bar_clear_buttons (path_bar);
       path_bar->button_list = g_list_reverse (new_buttons);
+      path_bar->fake_root = fake_root;
 
       for (l = path_bar->button_list; l; l = l->next)
 	{
@@ -1144,9 +1392,7 @@ _gtk_path_bar_set_path (GtkPathBar         *path_bar,
 	  ButtonData *button_data;
 
 	  button_data = BUTTON_DATA (l->data);
-
-	  gtk_widget_unref (button_data->button);
-	  button_data_free (button_data);
+	  gtk_widget_destroy (button_data->button);
 	}
 
       g_list_free (new_buttons);
@@ -1157,7 +1403,6 @@ _gtk_path_bar_set_path (GtkPathBar         *path_bar,
   return result;
 }
 
-
 /* FIXME: This should be a construct-only property */
 void
 _gtk_path_bar_set_file_system (GtkPathBar    *path_bar,
